==> ada-cli-1.1.0.tgz: package/lib/api.js <==
'use strict'

const _ = require('lodash')
const request = require('request')
// require('request-debug')(request)

const config = require('../config.json')
const conf = config[process.env.NODE_ENV || 'dev']
const auth = conf.medx.auth
const c3 = conf.medx.c3

const no = ['no', 'none', 'nope', 'nein']
const abortSearch = (term) => _.includes(no, term.trim().toLowerCase())

module.exports = (username, password) => {
  let api = null

  const loginFormData = {
    username: username,
    password: password
  }

  const initialFormData = JSON.stringify({
    meta: { state: 'DASHBOARD' },
    answer: { key: 'GET_HEALTH_ADVICE' }
  })

  const getResult = (resultURL, cb) => {
    api.get({url: resultURL}, (err, res, body) => {
      if (err) cb(err)
      cb(null, JSON.parse(body))
    })
  }

  const normalizeResponse = (response) => {
    const firstType = _.get(response, 'answerOptions[0].type')
    const secondType = _.get(response, 'answerOptions[1].type')

    response.answerType = firstType === 'SELECT' ? 'SELECT' : 'TEXT'
    response.search = firstType === 'SEARCH'

    if (firstType === 'SEARCH' && secondType) {
      response.question += ' Type "no" or enter a symptom.'
    }
    return response
  }

  const symptomSearch = (answer, previousMessage, cb) => {
    api.post({
      url: `/dialog/search?limit=5&offset=0&query=${encodeURIComponent(answer)}`,
      body: JSON.stringify({meta: previousMessage.meta})},
      (err, res, body) => {
        const newMessage = {
          question: 'Which descibes your pain best?',
          answerType: 'SELECT',
          meta: previousMessage.meta,
          answerOptions: JSON.parse(body).results
        }

        cb(err, !err && newMessage)
      })
  }

  return {
    login: (cb) => {
      request.post({url: auth, formData: loginFormData}, (err, res) => {
        if (err) cb(err)
        const cookie = res.headers['set-cookie'][0].match(/([^=]+=[^;]+)/)[1]

        api = request.defaults({
          baseUrl: c3,
          headers: {
            cookie,
            'Content-Type': 'application/json;charset=UTF-8'
          }
        })

        api.put({url: '/dialog', body: initialFormData}, (err, res, body) => {
          cb(err, !err && normalizeResponse(JSON.parse(body)))
        })
      })
    },

    isLoggedIn: () => !!api,

    ask: (answer, previousMessage, cb) => {
      if (!api) cb(new Error('wrong function'))

      if (previousMessage.search && !abortSearch(answer)) {
        return symptomSearch(answer, previousMessage, cb)
      }

      let dialogFormData = {
        meta: previousMessage.meta,
        answer: answer
      }

      if (previousMessage.answerType === 'TEXT') {
        answer = answer.trim()
        dialogFormData.answer = abortSearch(answer) ? {key: 'NO'}
        : {key: answer, text: answer}
      }

      if (answer.score) dialogFormData.answerType = 'SEARCH'

      dialogFormData = JSON.stringify(dialogFormData)

      api.put({url: '/dialog', body: dialogFormData}, (err, res, body) => {
        body = JSON.parse(body)
        if (body.meta && body.meta.instantRedirect) {
          return getResult(body.meta.instantRedirect, cb)
        }
        cb(err, !err && normalizeResponse(body))
      })
    }
  }
}

==> ada-cli-1.2.2.tgz: package/lib/api.js <==
'use strict'

const _ = require('lodash')
const request = require('request')
// require('request-debug')(request)

const config = require('./rc')().get()
const auth = config.medx.auth
const c3 = config.medx.c3

const no = ['no', 'none', 'nope', 'nein']
const abortSearch = (term) => _.includes(no, term.trim().toLowerCase())

module.exports = (username, password) => {
  let api = null

  const loginFormData = {
    username: username,
    password: password
  }

  const initialFormData = JSON.stringify({
    meta: { state: 'DASHBOARD' },
    answer: { key: 'GET_HEALTH_ADVICE' }
  })

  const getResult = (resultURL, cb) => {
    api.get({url: resultURL}, (err, res, body) => {
      if (err) cb(err)
      cb(null, JSON.parse(body))
    })
  }

  const normalizeResponse = (response) => {
    const firstType = _.get(response, 'answerOptions[0].type')
    const secondType = _.get(response, 'answerOptions[1].type')

    response.answerType = firstType === 'SELECT' ? 'SELECT' : 'TEXT'
    response.search = firstType === 'SEARCH'

    if (firstType === 'SEARCH' && secondType) {
      response.question += ' Type "no" or enter a symptom.'
    }
    return response
  }

  const symptomSearch = (answer, previousMessage, cb) => {
    api.post({
      url: `/dialog/search?limit=5&offset=0&query=${encodeURIComponent(answer)}`,
      body: JSON.stringify({meta: previousMessage.meta})},
      (err, res, body) => {
        const newMessage = {
          question: 'Which descibes your pain best?',
          answerType: 'SELECT',
          meta: previousMessage.meta,
          answerOptions: JSON.parse(body).results
        }

        cb(err, !err && newMessage)
      })
  }

  return {
    login: (cb) => {
      request.post({url: auth, formData: loginFormData}, (err, res) => {
        if (err) cb(err)
        const cookie = res.headers['set-cookie'][0].match(/([^=]+=[^;]+)/)[1]

        api = request.defaults({
          baseUrl: c3,
          headers: {
            cookie,
            'Content-Type': 'application/json;charset=UTF-8'
          }
        })

        api.put({url: '/dialog', body: initialFormData}, (err, res, body) => {
          cb(err, !err && normalizeResponse(JSON.parse(body)))
        })
      })
    },

    isLoggedIn: () => !!api,

    ask: (answer, previousMessage, cb) => {
      if (!api) cb(new Error('wrong function'))

      if (previousMessage.search && !abortSearch(answer)) {
        return symptomSearch(answer, previousMessage, cb)
      }

      let dialogFormData = {
        meta: previousMessage.meta,
        answer: answer
      }

      if (previousMessage.answerType === 'TEXT') {
        answer = answer.trim()
        dialogFormData.answer = abortSearch(answer) ? {key: 'NO'}
        : {key: answer, text: answer}
      }

      if (answer.score) dialogFormData.answerType = 'SEARCH'

      dialogFormData = JSON.stringify(dialogFormData)

      api.put({url: '/dialog', body: dialogFormData}, (err, res, body) => {
        body = JSON.parse(body)
        if (body.meta && body.meta.instantRedirect) {
          return getResult(body.meta.instantRedirect, cb)
        }
        cb(err, !err && normalizeResponse(body))
      })
    }
  }
}
